#!/usr/bin/env python3
"""Analyze Mbias"""
import os
import pathlib
import sys
from argparse import ArgumentParser
from pathlib import Path

import matplotlib.pyplot as plt
import numpy as np
from vdaq_ana import ShowProgress

from vdaq_soap.data_utils import open_data_file


def read_hold_delay_data(fname, mod_id, emin=-1, document=None):
    """Read the scan data."""
    print(":> Opening {} for hold_delay analysis".format(fname))
    vdaq = open_data_file(fname)
    if vdaq is None:
        return

    # get the iterator of the Scan data
    scan_iter = vdaq.scan_iter()
    scan_point = next(scan_iter)
    point_values = [scan_point.values[11]]
    point_mean = []
    point_data = []

    prg = ShowProgress(vdaq.nevts, width=24)
    prg.start()
    for evt in vdaq:
        # Get the module id
        mid = evt.mod_id

        if evt.evt_time > scan_point.end:
            # Next point
            try:
                scan_point = next(scan_iter)
                point_values.append(scan_point.values[11])

                if len(point_data):
                    point_mean.append(np.mean(point_data))
                else:
                    point_mean.append(0.0)

                point_data = []

            except StopIteration:
                print("Stop iteration")
                print(evt.evt_time, scan_point.end)
                break

        if mid != mod_id:
            continue

        md = vdaq.modules[mid]
        data = md.process_event(evt)
        if data is not None:
            for C, E in data:
                if E > emin:
                    point_data.append(E)

        prg.increase(show=True)

    print("")
    point_mean.append(np.mean(point_data))

    imax = np.argmax(point_mean)
    vmax = point_mean[imax]
    hd_max = point_values[imax]
    print("\nMaximum is {:.1f} at HD={}".format(vmax, hd_max))

    fig, ax = plt.subplots(1, 1)
    ax.plot(point_values, point_mean, 'o-')
    ax.set_title("Hold delay scan")
    ax.set_xlabel("Hold Delay (dac)")
    pdf_file = pathlib.Path.cwd().joinpath("hold_delay_{}.png".format(mod_id))
    fig.savefig(pdf_file, dpi=300)
    if document:
        document.add_heading('Hold Delay Scan', level=2)
        p = document.add_paragraph("Maximum is {:.1f} at HD={}".format(point_mean[imax], point_values[imax]))
        document.add_picture(str(pdf_file))
        os.remove(pdf_file)

    plt.draw()
    plt.pause(0.001)
    return int(hd_max)


def analyzeHoldDelay(files, options):
    """main entry."""
    ifile = Path(files[0]).expanduser().resolve()
    if not ifile.exists():
        return

    read_hold_delay_data(ifile, options.mid, options.emin, None)
    plt.show()


def do_analyzeHoldDelay():
    """Main entry."""
    parser = ArgumentParser()
    parser.add_argument('files', nargs='*', help="Input files")
    parser.add_argument("--emin", default=0.0, type=float, help="Min E to show in histogram")

    parser.add_argument("--mid", dest="mid", default=11, type=int, help="The slot ID")

    options = parser.parse_args()

    if len(options.files) == 0:
        print("I need an input file")
        sys.exit()

    analyzeHoldDelay(options.files, options)


if __name__ == "__main__":
    do_analyzeHoldDelay()


#!/usr/bin/env python3
"""Analyze Mbias"""
import os
import pathlib
import sys
from argparse import ArgumentParser
from pathlib import Path

import matplotlib.pyplot as plt
import numpy as np
from vdaq_ana import ShowProgress

from vdaq_soap.data_utils import open_data_file


def read_mbias_scan(fname, mod_id, emin=-1, document=None):
    """Read the mbias scan data.

    Args:
    ----
        fname: The file name
        mod_id: The modile ID
        document (optional): The word document. Defaults to None.

    """
    print(":> Opening {} for mbias analysis".format(fname))
    vdaq = open_data_file(fname)
    if vdaq is None:
        return

    # get the iterator of the Scan data
    scan_iter = vdaq.scan_iter()
    scan_point = next(scan_iter)
    point_values = [scan_point.values[4]]
    point_mean = []
    point_rms = []
    point_data = []
    point_good = []
    good_chan = 0
    n_evts = 0

    try:
        the_chan = scan_point.values[0]
    except KeyError:
        the_chan = -1

    prg = ShowProgress(vdaq.nevts, width=24)
    prg.start()
    for evt in vdaq:
        # Get the module id
        mid = evt.mod_id

        if evt.evt_time > scan_point.end:
            # Next point
            try:
                scan_point = next(scan_iter)
                point_values.append(scan_point.values[4])
                if n_evts:
                    point_good.append(good_chan/n_evts)
                else:
                    point_good.append(0)

                if len(point_data) > 3:
                    point_mean.append(np.mean(point_data))
                    point_rms.append(np.std(point_data))
                else:
                    point_mean.append(0)
                    point_rms.append(0)

                try:
                    the_chan = scan_point.values[0]
                except KeyError:
                    the_chan = -1

                n_evts = 0
                good_chan = 0
                point_data = []

            except StopIteration:
                print("Stop iteration")
                print(evt.evt_time, scan_point.end)
                break

        if mid != mod_id:
            continue

        n_evts += 1
        md = vdaq.modules[mid]
        data = md.process_event(evt)
        if data is not None:
            ng = 0
            for C, E in data:
                if C == the_chan:
                    ng += 1

                if E > emin:
                    point_data.append(E)

            if ng > 0:
                good_chan += 1

        prg.increase(show=True)

    print("")
    point_mean.append(np.mean(point_data))
    point_rms.append(np.std(point_data))
    if n_evts:
        point_good.append(good_chan/n_evts)
    else:
        point_good.append(0)

    imax = np.argmax(point_mean)
    imin = np.argmin(point_rms)
    print("\nMaximum amplitud is {:.1f} at Mbias={:.1f}".format(point_mean[imax], point_values[imax]))
    print("Minimum rms is {:.1f} at Mbias={:.1f}".format(point_rms[imin], point_rms[imin]))

    fig, ax = plt.subplots(3, 1, figsize=(6.4, 7.2), tight_layout=True)

    ax[0].plot(point_values, point_good, 'o-')
    ax[0].set_title("Mbias scan - good channel")
    ax[0].set_xlabel("mbias")

    ax[1].plot(point_values, point_mean, 'o-')
    ax[1].set_title("Mbias scan - Amplitude")
    ax[1].set_xlabel("mbias")
    ax[2].plot(point_values, point_rms, 'o-')
    ax[2].set_title("Mbias scan - noise")
    ax[2].set_xlabel("mbias")

    pdf_file = pathlib.Path.cwd().joinpath("mbias_{}.png".format(mod_id))
    fig.savefig(pdf_file, dpi=300)
    if document:
        document.add_heading('Mbias Scan', level=2)
        p = document.add_paragraph("Maximum amplitud is {:.1f} at Mbias={:.1f}".format(point_mean[imax],
                                                                                       point_values[imax]))
        p = document.add_paragraph("Minimum rms is {:.1f} at Mbias={:.1f}".format(point_rms[imin],
                                                                                  point_rms[imin]))
        document.add_picture(str(pdf_file))
        os.remove(pdf_file)

    plt.draw()
    plt.pause(0.001)


def analyzeMbias(files, options):
    """main entry."""
    ifile = Path(files[0]).expanduser().resolve()
    if not ifile.exists():
        return

    read_mbias_scan(ifile, options.mid, options.emin, None)
    plt.show()


def do_analyzeMbias():
    """Main entry."""
    parser = ArgumentParser()
    parser.add_argument('files', nargs='*', help="Input files")
    parser.add_argument("--emin", default=0.0, type=float, help="Min E to show in histogram")

    parser.add_argument("--mid", dest="mid", default=11, type=int, help="The slot ID")

    options = parser.parse_args()

    if len(options.files) == 0:
        print("I need an input file")
        sys.exit()

    analyzeMbias(options.files, options)


if __name__ == "__main__":
    do_analyzeMbias()


#!/usr/bin/env python3
"""Do a Hold delay scan."""
import sys
from argparse import ArgumentParser
import numpy as np
import daqpp.soap
import matplotlib.pyplot as plt

from vdaq_soap.soapCheckAliVATA import set_module_param
from vdaq_soap.soapCheckAliVATA import wait_for_end_of_run, get_monitor_data

def remove_outliers(data, cut=2.0, outliers=False, debug=False):
    """Remove points far away form the rest.

    Args:
    ----
        data : The data
        cut: max allowed distance
        outliers: if True, return the outliers rhater than remove them
        debug: be verbose if True.

    """
    d = np.abs(data - np.median(data))
    mdev = np.median(d)
    s = d / (mdev if mdev else 1.)
    if outliers:
        indx = np.where(s > cut)[0]
    else:
        indx = np.where(s < cut)[0]

    return indx

def maskNoisyChannels(server, cut=7.5, do_run=False):
    """Main Entry."""

    if do_run:
        # Stop the run if running
        print("Check if the RUnManager is active and running")
        run_manager = daqpp.soap.RunManager("main", server)
        S = run_manager.getStatus()
        if S.status == "Running":
            print("Stopping run")
            run_manager.stopRun()

        # Create a new scan run
        server.createRunManager("pedestal", "main")
        run_manager = daqpp.soap.RunManager("main", server)
        set_module_param(run_manager, "ro_mode", 1)

        # Reset all monitor data
        for the_module in server.getAllModules():
        	the_module.resetMonitorData()

        run_manager.setMaxEvents(5000)
        run_manager.startRun()
        wait_for_end_of_run(run_manager, 5.0)

    # Get the module
    the_module = None
    modules = server.getAllModules()
    for the_module in modules:
        print("Module {}".format(the_module.name))



        # Get the data
        data = get_monitor_data(the_module)
        noise =  data['histograms']['noise']
        channels = remove_outliers(noise['data'], cut, outliers=True)

        chanlist = ""
        for chan in channels:
            chanlist += "{},".format(chan)

        try:
            if chanlist[-1]==',':
                chanlist = chanlist[:-1]

            the_module.getParameter("command").set_value("setMask|{}=1".format(chanlist))

        except IndexError:
            pass



            print(channels)

def main():
    """Entry Point."""
    parser = ArgumentParser()
    parser.add_argument("--host", dest="host", default="localhost",
                        type=str, help="The soap server")
    parser.add_argument("--port", dest="port", default=50000,
                        type=int, help="The soap port")
    parser.add_argument("--cut", dest="cut", default=7.5,
                        type=float, help="The cut to remove outliers")
    parser.add_argument("--run", action="store_true", default=False, help="Add to make a pedestal run.")

    options = parser.parse_args()

    # This is where we connect with the server
    try:
        server = daqpp.soap.DAQppClient(options.host, options.port)

    except Exception as E:
        print(("Could not connect to the server\n{}".format(E)))
        sys.exit()


    maskNoisyChannels(server, cut=options.cut, do_run=options.run)

if __name__ == "__main__":
    main()




from vdaq_soap.soapCheckAliVATA import soapCheckAliVATA
from vdaq_soap.soapHoldDelay import soapHoldDelay
from vdaq_soap.soapMbias import soapMbias
from vdaq_soap.soapTestChannel import soapTestChannel
from vdaq_soap.analyzeHoldDelay import do_analyzeHoldDelay
from vdaq_soap.analyzeMbias import do_analyzeMbias
from vdaq_soap.maskNoisyChannels import maskNoisyChannels

__version__ = "1.1.3"
